from .middlewares import simple_view_middleware, urls_middleware, GenericMiddleware
from .navigation import Destiny, Navigare
from .pages import GenericApp, GenericClient
from .views import ft_view, GenericViewFactory
